#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import atexit
import logging
logger = logging.getLogger('PyAMS (scheduler')

# import interfaces
from pyams_scheduler.interfaces import SCHEDULER_HANDLER_KEY, SCHEDULER_NAME
from pyams_utils.interfaces import PYAMS_APPLICATION_SETTINGS_KEY, PYAMS_APPLICATION_DEFAULT_NAME
from pyams_utils.interfaces.zeo import IZEOConnection
from zope.interface.interfaces import ComponentLookupError

# import packages
from pyams_scheduler.process import SchedulerProcess, SchedulerMessageHandler
from pyams_utils.zodb import get_connection_from_settings
from pyams_zmq.process import process_exit_func
from zmq.error import ZMQError


def include_package(config):
    """Pyramid package include"""

    # add translations
    config.add_translation_dirs('pyams_scheduler:locales')

    # load registry components
    try:
        import pyams_zmi
    except ImportError:
        config.scan(ignore='pyams_scheduler.zmi')
    else:
        config.scan()

    start_handler = config.registry.settings.get(SCHEDULER_HANDLER_KEY, False)
    if start_handler:
        # create scheduler process
        process = SchedulerProcess(start_handler, SchedulerMessageHandler, config.registry)
        # get database connection
        connection = get_connection_from_settings(config.registry.settings)
        root = connection.root()
        # get application
        application_name = config.registry.settings.get(PYAMS_APPLICATION_SETTINGS_KEY,
                                                        PYAMS_APPLICATION_DEFAULT_NAME)
        application = root.get(application_name)
        if application is not None:
            sm = application.getSiteManager()
            scheduler_util = sm.get(SCHEDULER_NAME)
            try:
                zeo_connection_name = scheduler_util.zeo_connection
            except ComponentLookupError:
                pass
            else:
                zeo_connection = sm.getUtility(IZEOConnection, name=zeo_connection_name)
                # load tasks
                for task in scheduler_util.values():
                    trigger = task.get_trigger(config.registry)
                    logger.debug("Adding scheduler job for task '{0.name}'".format(task))
                    process.scheduler.add_job(task, trigger,
                                              id=str(task.internal_id),
                                              name=task.name,
                                              kwargs={'zeo_settings': zeo_connection.get_settings(),
                                                      'registry': config.registry})
                logger.debug("Starting tasks scheduler {0!r}".format(process))
                # start process
                try:
                    process.start()
                except ZMQError:
                    process.join()
                else:
                    if process.is_alive():
                        atexit.register(process_exit_func, process=process)
